// Copyright 2016 Orus.io. All rights reserved.

package cmd

import (
	"os"

	"github.com/m4rw3r/uuid"
	"github.com/olekukonko/tablewriter"
	"github.com/spf13/cobra"
)

var psCmd = &cobra.Command{
	Use:   "ps",
	Short: "Process list",
	Run: func(cmd *cobra.Command, args []string) {
		c, err := GetClient()
		if err != nil {
			return
		}
		processList, err := c.ProcessQuery(false)
		if err != nil {
			Logger.Errorf("Error: %s", err)
		}
		table := tablewriter.NewWriter(os.Stdout)
		table.SetHeader([]string{"id", "graph", "emitter", "status"})
		for _, p := range processList {
			table.Append([]string{
				p.ID.String(),
				p.GraphID.String(),
				p.TriggerEmitterID.String(),
				p.Status.String(),
			})
		}
		table.Render()
	},
}

var psPauseCmd = &cobra.Command{
	Use:   "pause",
	Short: "Pause a process",
	Run: func(cmd *cobra.Command, args []string) {
		os.Exit(doCommand(args[0], "pause"))
	},
}
var psResumeCmd = &cobra.Command{
	Use:   "resume",
	Short: "Resume a process",
	Run: func(cmd *cobra.Command, args []string) {
		os.Exit(doCommand(args[0], "resume"))
	},
}

func doCommand(id, command string) int {
	c, err := GetClient()
	if err != nil {
		return 1
	}
	pID, err := uuid.FromString(id)
	if err != nil {
		Logger.Errorf("Invalid uuid: %s", id)
		return 1
	}
	err = c.ProcessControl(pID, "pause")
	if err != nil {
		Logger.Errorf("Error: %s", err)
		return 1
	}
	return 0
}

func init() {
	RootCmd.AddCommand(psCmd)
	psCmd.AddCommand(psPauseCmd)
	psCmd.AddCommand(psResumeCmd)
}
