// Copyright 2016 Orus.io. All rights reserved.

package cmd

import (
	"github.com/slicebit/qb"
	"github.com/spf13/cobra"
	"github.com/spf13/viper"

	"bitbucket.org/orus-io/go-xbus"
	"bitbucket.org/orus-io/xbus/storage/sql/model"
)

// initCmd represents the init command
var initCmd = &cobra.Command{
	Use:   "init",
	Short: "Initialize the Xbus database.",
	Long: `This command can be used to automatically create or update Xbus
configuration tables in a PostgreSQL database.`,
	Run: initXbus,
}

func init() {
	RootCmd.AddCommand(initCmd)
}

func initXbus(cmd *cobra.Command, args []string) {
	// during init we log all... TODO: Should we take args like others?
	opts := xbus.LoggerOptions{Logtime: true, Debug: true, Trace: true}
	logger := xbus.GetLogger(&opts)

	m, db, err := model.NewDB("postgres", viper.GetString("database"), Logger, qb.LDefault)
	if err != nil {
		logger.Fatalf("Failed to initialize db access: %s", err)
	}
	if err := m.Meta.GetQbMetadata().CreateAll(db.Engine); err != nil {
		logger.Fatalf("Failed to create tables: %s", err)
	}
}
