package server

import (
	"bitbucket.org/orus-io/xbus/services"
	"bitbucket.org/orus-io/xbus/services/broker"
	"bitbucket.org/orus-io/xbus/services/control"
	"bitbucket.org/orus-io/xbus/services/core"
	"bitbucket.org/orus-io/xbus/services/emitter"
	"bitbucket.org/orus-io/xbus/services/processstate"
	"bitbucket.org/orus-io/xbus/services/registration"
	"bitbucket.org/orus-io/xbus/services/sentinel"
)

func (s *Server) servicesOptions() services.Options {
	return services.Options{
		Conn:   s.natsConn,
		Logger: s.log,
	}
}

func (s *Server) initServiceBroker() *broker.Service {
	return broker.New(broker.Options{
		Options:         s.servicesOptions(),
		ActorStorage:    s.actorStorage,
		EnvelopeStorage: s.envelopeStorage,
		LogStorage:      s.logStorage,
		ProcessManager:  s.processManager,
	})
}

func (s *Server) initServiceControl() *control.Service {
	return control.NewService(
		control.Options{
			Options:        s.servicesOptions(),
			ServerCert:     s.clientCA,
			ServerKey:      s.clientCAKey,
			GraphManager:   s.graphManager,
			ProcessStorage: s.processStorage,
			AccountStorage: s.accountStorage,
			ActorStorage:   s.actorStorage,
		})
}

func (s *Server) initServiceCore() *core.Service {
	return core.Options{
		Options:        s.servicesOptions(),
		ActorStorage:   s.actorStorage,
		ProcessStorage: s.processStorage,
	}.NewService()
}

func (s *Server) initServiceEmitter() *emitter.Service {
	opts := emitter.Options{
		Options:         s.servicesOptions(),
		ActorStorage:    s.actorStorage,
		EnvelopeStorage: s.envelopeStorage,
		ProcessManager:  s.processManager,
	}
	if s.options.EmitterReceptionTimeout != 0 {
		opts.ReceptionTimeout = s.options.EmitterReceptionTimeout
	}
	return opts.NewService()
}

func (s *Server) initServiceProcessState() *processstate.Service {
	return processstate.Options{
		Options:        s.servicesOptions(),
		LogStorage:     s.logStorage,
		ProcessManager: s.processManager,
	}.NewService()
}

func (s *Server) initServiceRegistration() *registration.Service {
	return registration.NewService(
		registration.Options{
			Options:        s.servicesOptions(),
			AccountStorage: s.accountStorage,
			ActorStorage:   s.actorStorage,
			ServerCA:       s.serverCert.Raw,
		})
}

func (s *Server) initServiceSentinel() *sentinel.Service {
	return sentinel.Options{
		Options:      s.servicesOptions(),
		ActorStorage: s.actorStorage,
	}.NewService()
}

func (s *Server) setupServices() {
	s.ServiceManager = services.NewManager(s.log)

	s.ServiceManager.Add(s.initServiceCore())

	if s.options.ServeBroker {
		s.ServiceManager.Add(s.initServiceBroker())
	}

	if s.options.ServeControl {
		s.ServiceManager.Add(s.initServiceControl())
	}

	if s.options.ServeEmitter {
		s.ServiceManager.Add(s.initServiceEmitter())
	}

	if s.options.ServeProcessState {
		s.ServiceManager.Add(s.initServiceProcessState())
	}

	if s.options.ServeRegistration {
		s.ServiceManager.Add(s.initServiceRegistration())
	}

	if s.options.ServeSentinel {
		s.ServiceManager.Add(s.initServiceSentinel())
	}
}
