// Copyright 2016 Orus.io. All rights reserved.

package control

import (
	"fmt"

	"github.com/m4rw3r/uuid"

	"bitbucket.org/orus-io/go-xbus/api"
)

func (s *Service) actorQuery(filters []api.Actor) ([]api.Actor, error) {
	return s.o.ActorStorage.List(), nil
}

func (s *Service) actorAccept(id uuid.UUID) (api.Actor, error) {
	a, exists := s.o.ActorStorage.Get(id)
	if !exists {
		return api.Actor{}, fmt.Errorf("No such actor: %s", id)
	}
	if a.Status != api.ActorPending {
		return api.Actor{}, fmt.Errorf("Invalid initial status '%s' for 'Accept'", a.Status)
	}
	a.Status = api.ActorEnabled
	return s.o.ActorStorage.Update(a)
}

func (s *Service) actorReject(id uuid.UUID) (api.Actor, error) {
	a, exists := s.o.ActorStorage.Get(id)
	if !exists {
		return api.Actor{}, fmt.Errorf("No such actor: %s", id)
	}
	if a.Status != api.ActorPending {
		return api.Actor{}, fmt.Errorf("Invalid initial status '%s' for 'Reject'", a.Status)
	}
	a.Status = api.ActorRejected
	return s.o.ActorStorage.Update(a)
}

func (s *Service) handleActorCRUD(subject, reply string, req api.ControlActorRequest) {
	action := subject[len(api.ControlActorsPrefix):]
	var res api.ControlActorResponse
	s.TaskAdd()
	defer s.SendReply(reply, &res)
	var err error
	switch action {
	case api.ActionUpdate:
		for _, actor := range req.Actors {
			var sActor api.Actor
			if actor.Status == api.ActorEnabled {
				sActor, err = s.actorAccept(actor.ID)
			} else if actor.Status == api.ActorRejected {
				sActor, err = s.actorReject(actor.ID)
			} else {
				res.Response = api.ClientError("Invalid actor update")
				return
			}
			actor = api.Actor{
				ID:     sActor.ID,
				Name:   sActor.Name,
				Kind:   sActor.Kind,
				Status: sActor.Status,
				Roles:  sActor.Roles,
			}

			if err != nil {
				break
			}
			res.Actors = append(res.Actors, actor)
		}
	case api.ActionDelete:
	case api.ActionQuery:
		res.Actors, err = s.actorQuery(req.Actors)
	default:
		s.Conn.Publish(reply, api.ClientError("Invalid action: %s", action))
		return
	}

	if err != nil {
		s.Log.Errorf("%s", err)
		res.Response = api.ServerError(err)
	} else {
		res.Status = api.StatusOK
	}
}
