// Copyright 2016 Orus.io. All rights reserved.

// Package core provides the 'core' service, which must be started once and only once
// in every server process
package core

import (
	"bitbucket.org/orus-io/go-xbus/api"
	"bitbucket.org/orus-io/xbus/services"
	"bitbucket.org/orus-io/xbus/storage"
)

// Options is used to create a 'core' service
type Options struct {
	services.Options

	ActorStorage   storage.ActorStorage
	ProcessStorage storage.ProcessStorage
}

// NewService creates a Emitter
func (o Options) NewService() *Service {
	return &Service{
		BaseService: services.NewBaseService("core", o.Options),
		o:           o,
	}
}

// Service is in charge of emitting envelopes to Actors
type Service struct {
	services.BaseService

	o Options
}

// Startup starts the service
func (s *Service) Startup() error {
	s.Log.Noticef("Startup")
	s.o.ActorStorage.SetEventHandlers(s.onActorUp, s.onActorDown)
	s.o.ProcessStorage.SetEventHandlers(s.onProcessStatusChange)
	return nil
}

// Shutdown stops the service
func (s *Service) Shutdown() error {
	s.o.ActorStorage.SetEventHandlers(nil, nil)
	s.o.ProcessStorage.SetEventHandlers(nil)
	return nil
}

func (s *Service) onActorUp(actor api.Actor) {
	s.Conn.Publish(api.CoreActorEventsSubject, api.CoreEvent{
		Name: "up",
		Actor: api.Actor{
			ID:    actor.ID,
			Name:  actor.Name,
			Kind:  actor.Kind,
			Roles: actor.Roles,
		},
	})
}

func (s *Service) onActorDown(actor api.Actor) {
	s.Conn.Publish(api.CoreActorEventsSubject, api.CoreEvent{
		Name: "down",
		Actor: api.Actor{
			ID:    actor.ID,
			Name:  actor.Name,
			Kind:  actor.Kind,
			Roles: actor.Roles,
		},
	})
}

func (s *Service) onProcessStatusChange(process storage.Process, oldStatus api.ProcessStatus, status api.ProcessStatus) {
	if err := s.Conn.Publish(
		api.CoreProcessEventsSubject,
		api.CoreEvent{
			Name: "status",
			Process: api.Process{
				ID:                process.ID,
				TriggerEmitterID:  process.TriggerEmitterID,
				TriggerEnvelopeID: process.TriggerEnvelopeID,
				OldStatus:         oldStatus,
				Status:            status,
			},
		},
	); err != nil {
		s.Log.Errorf("Error publishing ProcessEvent: %s", err)
	}
}
