package services

import (
	"testing"
	"time"

	"github.com/nats-io/go-nats"
	"github.com/stretchr/testify/assert"

	"bitbucket.org/orus-io/go-xbus/api"
	"bitbucket.org/orus-io/xbus/testutils"
)

func TestSendReply(t *testing.T) {
	logger := testutils.GetTestLogger(t)
	srv := testutils.InitNatsTestServer(t, logger, nil, nil, nil)
	defer srv.Shutdown()
	c, err := nats.NewEncodedConn(srv.Connect(t), "json")
	assert.Nil(t, err)
	defer c.Close()

	s := NewBaseService("test", Options{Conn: c, Logger: logger})

	testResponses := make(chan api.Response)
	c.BindRecvChan("test", testResponses)

	t.Run("simple reply", func(t *testing.T) {
		res := api.Response{Status: api.StatusOK}
		s.TaskAdd()
		s.SendReply("test", &res)

		timeout := testutils.TimeoutChan(time.Second)
		select {
		case <-timeout:
			t.Errorf("No message received from SendReply")
		case r := <-testResponses:
			assert.Equal(t, api.StatusOK, r.Status)
		}
	})

	t.Run("panic", func(t *testing.T) {
		logger.ExpectsError(testutils.ExpectsContains("[test] Caught error: Craaaaash"))
		func() {
			res := api.Response{Status: api.StatusOK}
			s.TaskAdd()
			defer s.SendReply("test", &res)
			panic("Craaaaash")
		}()
		timeout := testutils.TimeoutChan(time.Second)
		select {
		case <-timeout:
			t.Errorf("No message received from SendReply")
		case r := <-testResponses:
			assert.Equal(t, api.StatusServerError, r.Status)
		}
		logger.VerifyExpectations()
	})

	t.Run("error replying", func(t *testing.T) {
		logger.ExpectsError("[test] Error replying: nats: connection closed")
		s.Conn.Close()
		res := api.Response{Status: api.StatusOK}
		s.TaskAdd()
		s.SendReply("test", &res)
		logger.VerifyExpectations()
	})
}
