// Copyright 2016 Orus.io. All rights reserved.

package model

import (
	"fmt"
	"time"

	"github.com/m4rw3r/uuid"
	"github.com/orus-io/yago"
)

//go:generate yago --fmt
//go:generate sed /m4rw3r/d -i base_yago.go

// Base struct for the Xbus broker database models.
//yago:notable,autoattrs
type Base struct {
	ID        uuid.UUID `yago:"primary_key"`
	CreatedAt time.Time
	UpdatedAt NullTime `yago:"type=qb.Timestamp()"`
}

// BeforeInsert callback
func (m *Base) BeforeInsert(db *yago.DB) {
	if m.ID.IsZero() {
		var err error
		m.ID, err = uuid.V4()
		if err != nil {
			panic(fmt.Sprintf("Could not generate a UUID. Got err: %s", err))
		}
	}
	m.CreatedAt = time.Now()
	m.UpdatedAt = NullTime{Time: time.Now()}
}

// BeforeUpdate callback
func (m *Base) BeforeUpdate(db *yago.DB) {
	m.UpdatedAt = NullTime{Time: time.Now()}
}
