// Copyright 2016 Orus.io. All rights reserved.

package model

import (
	"time"

	"github.com/m4rw3r/uuid"

	"bitbucket.org/orus-io/go-xbus/api"
)

//go:generate yago --fmt

// Process stores process data
//yago:
type Process struct {
	Base
	TriggerEmitterID  uuid.UUID `yago:"fk=Actor"`
	TriggerEnvelopeID uuid.UUID `yago:"fk=Envelope"`
	GraphID           uuid.UUID `yago:"."`
	SourceNodeID      string    `yago:"."`
	SourceSlotID      string    `yago:"."`
}

// Emission track envelope emission to actors
//yago:
type Emission struct {
	ID           int64     `yago:"primary_key,auto_increment"`
	ProcessID    uuid.UUID `yago:"fk=Process,unique_index=emission"`
	EnvelopeID   uuid.UUID `yago:"fk=Envelope,unique_index=emission"`
	TargetActor  uuid.UUID `yago:"fk=Actor,unique_index=emission"`
	TargetNodeID string    `yago:"unique_index=emission"`
	TargetInput  string    `yago:"unique_index=emission"`
	Start        bool      `yago:"."`
	Complete     bool      `yago:"."`
	// TODO store in a json field, or an array of something
	EventPositions string `yago:"type=qb.Text()"`
}

// ProcessLogEntry stores storage.ProcessLogEntry
//yago:
type ProcessLogEntry struct {
	ID        int64     `yago:"primary_key,auto_increment"`
	Timestamp time.Time `yago:"."`
	ProcessID uuid.UUID `yago:"fk=Process ONDELETE CASCADE"`
	Data      string    `yago:"type=qb.Text()"`
}

// ProcessState stores a storage.ProcessState
//yago:
type ProcessState struct {
	ProcessID  uuid.UUID         `yago:"primary_key,fk=Process"`
	Status     api.ProcessStatus `yago:"textmarshaled"`
	NodeStates string            `yago:"type=qb.Text()"`
}
