// Copyright 2017 Orus.io. All rights reserved.

package xbus

import (
	"fmt"
	"time"

	"bitbucket.org/orus-io/go-xbus/api"
)

// ActorLogger provides a Actor-based logging api
type ActorLogger struct {
	log   Logger
	actor *Actor
}

func (al ActorLogger) addLogMessage(env *api.Envelope, level api.LogLevel, format string, fmtArgs ...interface{}) {
	text := fmt.Sprintf(format, fmtArgs...)
	switch level {
	case api.LogNotice, api.LogWarning:
		al.log.Noticef(text)
	case api.LogError:
		al.log.Errorf(text)
	}
	if err := al.actor.SendLogs(env,
		[]api.LogMessage{
			api.LogMessage{
				Time:  time.Now(),
				Level: level,
				Text:  text,
			},
		}); err != nil {
		al.log.Errorf("Could not send the logs to xbus. Got '%s'", err)
	}
}

// Noticef logs a notif message
func (al ActorLogger) Noticef(format string, fmtArgs ...interface{}) {
	al.addLogMessage(nil, api.LogNotice, format, fmtArgs...)
}

// EnvNoticef logs a envelope-related notif message
func (al ActorLogger) EnvNoticef(env *api.Envelope, format string, fmtArgs ...interface{}) {
	al.addLogMessage(env, api.LogNotice, format, fmtArgs...)
}

// Warningf logs a warning message
func (al ActorLogger) Warningf(format string, fmtArgs ...interface{}) {
	al.addLogMessage(nil, api.LogWarning, format, fmtArgs...)
}

// EnvWarningf logs a envelope-related warning message
func (al ActorLogger) EnvWarningf(env *api.Envelope, format string, fmtArgs ...interface{}) {
	al.addLogMessage(env, api.LogWarning, format, fmtArgs...)
}

// Errorf logs a error message
func (al ActorLogger) Errorf(format string, fmtArgs ...interface{}) {
	al.addLogMessage(nil, api.LogError, format, fmtArgs...)
}

// EnvErrorf logs a envelope-related error message
func (al ActorLogger) EnvErrorf(env *api.Envelope, format string, fmtArgs ...interface{}) {
	al.addLogMessage(env, api.LogError, format, fmtArgs...)
}

// SendLogs sends log messages to the bus
func (a *Actor) SendLogs(
	envelope *api.Envelope,
	messages []api.LogMessage,
) error {
	var req = api.ActorLogReq{Messages: messages}
	if envelope != nil {
		req.EnvelopeID = envelope.ID
		req.Context = envelope.Context
	}
	a.GetClient().Log.Debugf("Sending %+v", req)
	var rep api.Response
	if err := a.Client.GetConn().Request(api.ActorLog(a.ID), &req, &rep, 10*time.Second); err != nil {
		return err
	}
	if rep.Status != api.StatusOK {
		return wrapAPIError(rep)
	}
	return nil
}
