package api

import (
	"github.com/m4rw3r/uuid"
)

// AccountStatus is the status of an Account
//go:generate enum-marshaler -type AccountStatus -prefix AccountStatus -lower -tests
type AccountStatus int

const (
	// AccountStatusNone is the empty AccountStatus
	AccountStatusNone AccountStatus = iota
	// AccountStatusPending is when the account needs to be validated by an
	// administrator
	AccountStatusPending

	// AccountStatusEnabled is when the account is fully functionnal (it has
	// a valid certificate) and was validated by an administrator
	AccountStatusEnabled

	// AccountStatusDisabled is when the account was (temporarily) disabled
	// by an administrator
	AccountStatusDisabled
)

// AccountType ...
//go:generate enum-marshaler -type AccountType -prefix AccountType -lower -tests
type AccountType int

const (
	// AccountTypeNone is the empty AccountType
	AccountTypeNone AccountType = iota
	// AccountTypeActor is for accounts attached to actors
	AccountTypeActor

	// AccountTypeUser is for user accounts
	AccountTypeUser
)

// Account ...
type Account struct {
	ID     uuid.UUID     `json:"id,omitempty"`
	Name   string        `json:"name,omitempty"`
	Type   AccountType   `json:"type,omitempty"`
	Status AccountStatus `json:"status,omitempty"`
	CSR    string        `json:"csr,omitempty"`
	Cert   string        `json:"cert,omitempty"`
}

const (
	// ControlAccountPrefix is the prefix of control account apis
	ControlAccountPrefix = ControlPrefix + "account."

	// ControlAccountList returns a list of account
	ControlAccountList = ControlAccountPrefix + "list"
)

// ControlAccountListReq is the request of ControlAccountList
type ControlAccountListReq struct{}

// ControlAccountListRep is the response of ControlAccountList
type ControlAccountListRep struct {
	Response

	AccountList []Account
}

// ControlAccountUpdate is the subject for updating an account
const ControlAccountUpdate = ControlAccountPrefix + "update"

// ControlAccountUpdateReq is the request of ControlAccountUpdate
type ControlAccountUpdateReq struct {
	Account Account
}

// ControlAccountUpdateRep is the reply of ControlAccountUpdate
type ControlAccountUpdateRep struct {
	Response

	Account Account
}
