// Copyright 2016 Orus.io. All rights reserved.

package api

import (
	"time"

	"github.com/m4rw3r/uuid"
)

const (
	// ControlActorsPrefix prefix for control actors
	ControlActorsPrefix = ControlPrefix + "actors."
	// ControlActorsCreate api to create actors
	ControlActorsCreate = ControlActorsPrefix + ActionCreate
	// ControlActorsUpdate api to update actors
	ControlActorsUpdate = ControlActorsPrefix + ActionUpdate
	// ControlActorsDelete api to delete actors
	ControlActorsDelete = ControlActorsPrefix + ActionDelete
	// ControlActorsQuery api to retrive actors
	ControlActorsQuery = ControlActorsPrefix + ActionQuery
)

// ActorStatus ...
//go:generate enum-marshaler -type ActorStatus -prefix Actor -lower -tests
type ActorStatus int

const (
	// ActorPending is when the administrator needs to accept the actor
	ActorPending ActorStatus = iota
	// ActorEnabled is when the actor is enabled
	ActorEnabled
	// ActorDisabled is when the actor is disabled
	ActorDisabled
	// ActorRejected is when the actor was rejected by the administrator
	ActorRejected
)

// Actor is any actor of the bus
type Actor struct {
	ID        uuid.UUID   `json:"id,omitempty"`
	Name      string      `json:"name,omitempty"`
	Kind      ActorKind   `json:"kind,omitempty"`
	Status    ActorStatus `json:"status,omitempty"`
	AccountID uuid.UUID   `json:"account_id,omitempty"`
	Roles     []string    `json:"roles,omitempty" yaml:",flow"`
	Visible   bool        `json:"visible,omitempty"`
	LastSeen  time.Time   `json:"last_seen,omitempty"`
}

// ControlActorRequest is the arguments for actors CRUD
type ControlActorRequest struct {
	Actors []Actor
}

// ControlActorResponse is the response from actors CRUD
type ControlActorResponse struct {
	Response
	Actors []Actor
}
