package demo

import (
	"time"

	"bitbucket.org/orus-io/go-xbus"
)

func NewPrintToConsoleConsumer(actor *xbus.Actor) xbus.ActorService {
	return &PrintToConsoleConsumer{actor, nil}
}

type PrintToConsoleConsumer struct {
	actor *xbus.Actor
	sub   *xbus.Subscription
}

func (c *PrintToConsoleConsumer) Startup() error {
	sub, err := c.actor.Subscribe(func(input string, er *xbus.EnvelopeReader) {
		env, err := er.Complete(time.Second)
		if err != nil {
			c.actor.Log.Errorf("Error receiving: %s", err)
			return
		}
		c.actor.ProcessingSuccess(env)
		for _, event := range env.Events {
			for _, item := range event.Items {
				c.actor.GetClient().Log.Noticef(
					"print-to-console - %s: %s", event.Type, string(item))
			}
		}
	})
	if err != nil {
		return err
	}
	c.sub = sub
	return nil
}

func (c *PrintToConsoleConsumer) Shutdown() error {
	err := c.sub.Unsubscribe()
	c.sub = nil
	return err
}

func init() {
	xbus.RegisterActorService("demo.print-to-console", NewPrintToConsoleConsumer)
}
