package xbus

import (
	"errors"
	"time"

	"github.com/m4rw3r/uuid"

	"bitbucket.org/orus-io/go-xbus/api"
)

// SendEnvelope sends an envelope on a Actor output
func (c *Client) SendEnvelope(
	actorID uuid.UUID, output string, env api.Envelope,
) (status api.ReceptionStatus, err error) {
	var response api.EnvelopeAck

	subject := api.ActorOutputSubject(actorID, output)
	c.Log.Noticef("Sending envelope to %s", subject)
	err = c.conn.Request(subject, &env, &response, time.Second)
	if err != nil {
		c.Log.Errorf("Error sending envelope: %s", err)
		return
	}
	if response.ReceptionStatus == api.ReceptionError {
		err = errors.New(response.Reason)
		c.Log.Errorf("Error sending envelope: %s", err)
		return
	}
	return response.ReceptionStatus, nil
}
