package xbus

import (
	"time"

	"bitbucket.org/orus-io/go-xbus/api"
)

// EnvelopeReader is a reader api for envelope
// it takes care of validating the whole envelope
type EnvelopeReader struct {
	envelope api.Envelope
}

// NewEnvelopeReader initialize a EnvelopeReader
func NewEnvelopeReader(envelope api.Envelope) *EnvelopeReader {
	return &EnvelopeReader{
		envelope: envelope,
	}
}

// GetEvent returns an EventReader for an event of the envelope
func (r EnvelopeReader) GetEvent(index int) *EventReader {
	er := NewEventReader(r.envelope.Events[index])
	return er
}

// Complete returns the complete envelope once all parts were received
func (r *EnvelopeReader) Complete(timeout time.Duration) (*api.Envelope, error) {
	return &r.envelope, nil
}

// EventReader is a reader api for event
type EventReader struct {
	event api.Event

	currentIndex int
}

// NewEventReader initialize a EventReader
func NewEventReader(event api.Event) *EventReader {
	return &EventReader{
		event:        event,
		currentIndex: -1,
	}
}

// Complete waits for the event to be fully received
// and returns a complete event
// Use only on event that are always "small"
func (r *EventReader) Complete(timeout time.Duration) (*api.Event, error) {
	return &r.event, nil
}

// Next switch to the next item and returns true, or
// returns false if no item is available.
// In case of error, the EventReader 'Error' attribute is
// set to the last error
func (r *EventReader) Next() bool {
	// TODO this check is not adequate as ItemCount may be known only at
	// the last moment
	if r.currentIndex+1 < int(r.event.ItemCount) {
		r.currentIndex++
		return true
	}
	return false
}

// Item returns the current item. Next() must be called at least once.
func (r *EventReader) Item() api.Item {
	if r.currentIndex == -1 {
		panic("Next() must be called first")
	}
	return r.event.Items[r.currentIndex]
}
