package xbus

// ActorService is a service that implements a actor behavior
type ActorService interface {
	Startup() error
	Shutdown() error
}

// NewActorServiceFunc is a type of function that creates ActorService instances
type NewActorServiceFunc func(*Actor) ActorService

var actorServiceRegistry = make(map[string]NewActorServiceFunc)

// RegisterActorService registers a new ActorService factory function
func RegisterActorService(typename string, init NewActorServiceFunc) {
	_, ok := actorServiceRegistry[typename]
	if ok {
		panic("ActorService '" + typename + "' already registered")
	}
	actorServiceRegistry[typename] = init
}

// MakeActorService returns a new ActorService, or nil if not registered
func MakeActorService(typename string, actor *Actor) ActorService {
	f, ok := actorServiceRegistry[typename]
	if !ok {
		return nil
	}
	return f(actor)
}
